@@ -7,11 +7,22 @@ THERMIA_GENESIS_API_CONFIG_URL = "https://online-genesis.thermia.se/api/configur
 
 THERMIA_INSTALLATION_PATH = "/api/v1/Registers/Installations/"
 
+THERMIA_API_TYPE_CLASSIC = "classic"
+THERMIA_API_TYPE_GENESIS = "genesis"
+
 THERMIA_API_CONFIG_URLS_BY_API_TYPE = {
-    "classic": THERMIA_CLASSIC_API_CONFIG_URL,
-    "genesis": THERMIA_GENESIS_API_CONFIG_URL,
+    THERMIA_API_TYPE_CLASSIC: THERMIA_CLASSIC_API_CONFIG_URL,
+    THERMIA_API_TYPE_GENESIS: THERMIA_GENESIS_API_CONFIG_URL,
 }
 
+###############################################################################
+# Azure AD configuration
+###############################################################################
+
+THERMIA_AZURE_AUTH_URL = "https://thermialogin.b2clogin.com/thermialogin.onmicrosoft.com/b2c_1a_signuporsigninonline"
+THERMIA_AZURE_AUTH_CLIENT_ID_AND_SCOPE = "09ea4903-9e95-45fe-ae1f-e3b7d32fa385"
+THERMIA_AZURE_AUTH_REDIRECT_URI = "https://online-genesis.thermia.se/login"
+
 ###############################################################################
 # Register groups
 ###############################################################################
@@ -22,14 +33,6 @@ REG_GROUP_OPERATIONAL_TIME = "REG_GROUP_OPERATIONAL_TIME"
 REG_GROUP_OPERATIONAL_OPERATION = "REG_GROUP_OPERATIONAL_OPERATION"
 REG_GROUP_HOT_WATER = "REG_GROUP_HOT_WATER"
 
-REGISTER_GROUPS = [
-    REG_GROUP_TEMPERATURES,
-    REG_GROUP_OPERATIONAL_STATUS,
-    REG_GROUP_OPERATIONAL_TIME,
-    REG_GROUP_OPERATIONAL_OPERATION,
-    REG_GROUP_HOT_WATER,
-]
-
 ###############################################################################
 # Temperature registers
 ###############################################################################
@@ -60,6 +63,7 @@ REG_OPER_DATA_RETURN = "REG_OPER_DATA_RETURN"
 REG_DESIRED_SYS_SUPPLY_LINE_TEMP = "REG_DESIRED_SYS_SUPPLY_LINE_TEMP"
 REG_COOL_SENSOR_TANK = "REG_COOL_SENSOR_TANK"
 REG_COOL_SENSOR_SUPPLY = "REG_COOL_SENSOR_SUPPLY"
+REG_ACTUAL_POOL_TEMP = "REG_ACTUAL_POOL_TEMP"
 
 TEMPERATURE_REGISTERS = [
     REG_OUTDOOR_TEMPERATURE,
@@ -75,6 +79,7 @@ TEMPERATURE_REGISTERS = [
     REG_DESIRED_SYS_SUPPLY_LINE_TEMP,
     REG_COOL_SENSOR_TANK,
     REG_COOL_SENSOR_SUPPLY,
+    REG_ACTUAL_POOL_TEMP,
 ]
 
 ###############################################################################
