@@ -10,6 +10,7 @@ from ThermiaOnlineAPI.const import (
     OPERATIONAL_TIME_REGISTERS,
     REG_BRINE_IN,
     REG_BRINE_OUT,
+    REG_ACTUAL_POOL_TEMP,
     REG_COOL_SENSOR_SUPPLY,
     REG_COOL_SENSOR_TANK,
     REG_DESIRED_SUPPLY_LINE,
@@ -67,10 +68,8 @@ class ThermiaHeatPump:
 
     def update_data(self):
         self.__info = self.__api_interface.get_device_info(self.__device_id)
-        self.__status = self.__api_interface.get_device_status(
-            self.__device_id)
-        self.__device_data = self.__api_interface.get_device_by_id(
-            self.__device_id)
+        self.__status = self.__api_interface.get_device_status(self.__device_id)
+        self.__device_data = self.__api_interface.get_device_by_id(self.__device_id)
 
         self.__register_indexes["temperature"] = self.__status.get(
             "heatingEffectRegisters", [None, None]
@@ -80,8 +79,7 @@ class ThermiaHeatPump:
             self.__device_id
         )
         self.__group_operational_status = (
-            self.__api_interface.get__group_operational_status(
-                self.__device_id)
+            self.__api_interface.get__group_operational_status(self.__device_id)
         )
         self.__group_operational_time = (
             self.__api_interface.get__group_operational_time(self.__device_id)
@@ -113,9 +111,10 @@ class ThermiaHeatPump:
     def set_operation_mode(self, mode: str):
         self._LOGGER.info("Setting operation mode to " + str(mode))
 
-        self.__group_operational_operation[
-            "current"
-        ] = mode  # update local state before refetching data
+        if self.__group_operational_operation is not None:
+            self.__group_operational_operation[
+                "current"
+            ] = mode  # update local state before refetching data
         self.__api_interface.set_operation_mode(self, mode)
         self.update_data()
 
@@ -130,9 +129,63 @@ class ThermiaHeatPump:
         self.__api_interface.set_hot_water_switch_state(self, state)
         self.update_data()
 
+    def get_all_available_register_groups(self):
+        installation_profile_id = self.__info.get("installationProfileId")
+        register_groups = self.__api_interface.get_all_available_groups(
+            installation_profile_id
+        )
+
+        if register_groups is None:
+            return []
+
+        return list(map(lambda x: x["name"], register_groups))
+
+    def get_available_registers_for_group(self, register_group: str):
+        registers_for_group = self.__api_interface.get_register_group_json(
+            self.__device_id, register_group
+        )
+
+        if registers_for_group is None:
+            return []
+
+        return list(map(lambda x: x["registerName"], registers_for_group))
+
+    def get_register_data_by_register_group_and_name(
+        self, register_group: str, register_name: str
+    ):
+        register_group = self.__api_interface.get_register_group_json(
+            self.__device_id, register_group
+        )
+
+        if register_group is None:
+            self._LOGGER.error("No register group found for group: " + register_group)
+            return None
+
+        return self.__get_data_from_group_by_register_name(
+            register_group, register_name
+        )
+
+    def set_register_data_by_register_group_and_name(
+        self, register_group: str, register_name: str, value: int
+    ):
+        register_data = self.get_register_data_by_register_group_and_name(
+            register_group, register_name
+        )
+
+        if register_data is None:
+            self._LOGGER.error(
+                "No register group found for group: "
+                + register_group
+                + " and register: "
+                + register_name
+            )
+            return None
+
+        self.__api_interface.set_register_value(self, register_data["id"], value)
+        self.update_data()
+
     def __get_heat_temperature_data(self):
-        device_temperature_register_index = self.get_register_indexes()[
-            "temperature"]
+        device_temperature_register_index = self.get_register_indexes()["temperature"]
         if device_temperature_register_index is None:
             return None
 
@@ -178,12 +231,14 @@ class ThermiaHeatPump:
         data = [d for d in group if d["registerName"] == register_name]
 
         if len(data) != 1:
-            # Temperature status not supported
+            # Register not in the group
             return None
 
         data = data[0]
 
         return {
+            "id": data["registerIndex"],
+            "isReadOnly": data["isReadOnly"],
             "minValue": data["minValue"],
             "maxValue": data["maxValue"],
             "step": data["step"],
@@ -192,14 +247,12 @@ class ThermiaHeatPump:
 
     def __get_active_alarms(self):
         active_alarms = filter(
-            lambda alarm: alarm.get(
-                "isActiveAlarm", False) is True, self.__alarms
+            lambda alarm: alarm.get("isActiveAlarm", False) is True, self.__alarms
         )
         return list(active_alarms)
 
     def __set_historical_data_registers(self):
-        data = self.__api_interface.get_historical_data_registers(
-            self.__device_id)
+        data = self.__api_interface.get_historical_data_registers(self.__device_id)
 
         data_map = {}
 
@@ -287,11 +340,9 @@ class ThermiaHeatPump:
     @property
     def supply_line_temperature(self):
         return get_dict_value_safe(
-            self.__get_temperature_data_by_register_name(
-                REG_SUPPLY_LINE), "value"
+            self.__get_temperature_data_by_register_name(REG_SUPPLY_LINE), "value"
         ) or get_dict_value_safe(
-            self.__get_temperature_data_by_register_name(
-                REG_OPER_DATA_SUPPLY_MA_SA),
+            self.__get_temperature_data_by_register_name(REG_OPER_DATA_SUPPLY_MA_SA),
             "value",
         )
 
@@ -299,8 +350,7 @@ class ThermiaHeatPump:
     def desired_supply_line_temperature(self):
         return (
             get_dict_value_safe(
-                self.__get_temperature_data_by_register_name(
-                    REG_DESIRED_SUPPLY_LINE),
+                self.__get_temperature_data_by_register_name(REG_DESIRED_SUPPLY_LINE),
                 "value",
             )
             or get_dict_value_safe(
@@ -320,18 +370,21 @@ class ThermiaHeatPump:
     @property
     def return_line_temperature(self):
         return get_dict_value_safe(
-            self.__get_temperature_data_by_register_name(
-                REG_RETURN_LINE), "value"
+            self.__get_temperature_data_by_register_name(REG_RETURN_LINE), "value"
         ) or get_dict_value_safe(
-            self.__get_temperature_data_by_register_name(
-                REG_OPER_DATA_RETURN), "value"
+            self.__get_temperature_data_by_register_name(REG_OPER_DATA_RETURN), "value"
         )
 
     @property
     def brine_out_temperature(self):
         return get_dict_value_safe(
-            self.__get_temperature_data_by_register_name(
-                REG_BRINE_OUT), "value"
+            self.__get_temperature_data_by_register_name(REG_BRINE_OUT), "value"
+        )
+
+    @property
+    def pool_temperature(self):
+        return get_dict_value_safe(
+            self.__get_temperature_data_by_register_name(REG_ACTUAL_POOL_TEMP), "value"
         )
 
     @property
@@ -343,15 +396,13 @@ class ThermiaHeatPump:
     @property
     def cooling_tank_temperature(self):
         return get_dict_value_safe(
-            self.__get_temperature_data_by_register_name(
-                REG_COOL_SENSOR_TANK), "value"
+            self.__get_temperature_data_by_register_name(REG_COOL_SENSOR_TANK), "value"
         )
 
     @property
     def cooling_supply_line_temperature(self):
         return get_dict_value_safe(
-            self.__get_temperature_data_by_register_name(
-                REG_COOL_SENSOR_SUPPLY),
+            self.__get_temperature_data_by_register_name(REG_COOL_SENSOR_SUPPLY),
             "value",
         )
 
@@ -382,40 +433,35 @@ class ThermiaHeatPump:
     @property
     def compressor_operational_time(self):
         return get_dict_value_safe(
-            self.__get_operational_time_data_by_register_name(
-                REG_OPER_TIME_COMPRESSOR),
+            self.__get_operational_time_data_by_register_name(REG_OPER_TIME_COMPRESSOR),
             "value",
         )
 
     @property
     def hot_water_operational_time(self):
         return get_dict_value_safe(
-            self.__get_operational_time_data_by_register_name(
-                REG_OPER_TIME_HOT_WATER),
+            self.__get_operational_time_data_by_register_name(REG_OPER_TIME_HOT_WATER),
             "value",
         )
 
     @property
     def auxiliary_heater_1_operational_time(self):
         return get_dict_value_safe(
-            self.__get_operational_time_data_by_register_name(
-                REG_OPER_TIME_IMM1),
+            self.__get_operational_time_data_by_register_name(REG_OPER_TIME_IMM1),
             "value",
         )
 
     @property
     def auxiliary_heater_2_operational_time(self):
         return get_dict_value_safe(
-            self.__get_operational_time_data_by_register_name(
-                REG_OPER_TIME_IMM2),
+            self.__get_operational_time_data_by_register_name(REG_OPER_TIME_IMM2),
             "value",
         )
 
     @property
     def auxiliary_heater_3_operational_time(self):
         return get_dict_value_safe(
-            self.__get_operational_time_data_by_register_name(
-                REG_OPER_TIME_IMM3),
+            self.__get_operational_time_data_by_register_name(REG_OPER_TIME_IMM3),
             "value",
         )
 
@@ -467,8 +513,7 @@ class ThermiaHeatPump:
     @property
     def active_alarms(self):
         active_alarms = self.__get_active_alarms()
-        active_alarm_texts = map(
-            lambda alarm: alarm.get("eventTitle"), active_alarms)
+        active_alarm_texts = map(lambda alarm: alarm.get("eventTitle"), active_alarms)
         return list(active_alarm_texts)
 
     ###########################################################################
@@ -491,8 +536,7 @@ class ThermiaHeatPump:
         register_id = self.__historical_data_registers_map.get(register_name)
 
         if register_id is None:
-            self._LOGGER.error(
-                "Register name is not supported: " + str(register_name))
+            self._LOGGER.error("Register name is not supported: " + str(register_name))
             return None
 
         historical_data = self.__api_interface.get_historical_data(
@@ -531,15 +575,28 @@ class ThermiaHeatPump:
         print("########## DEBUG START ##########")
 
         print("self.__info:")
-        pretty_print_except(self.__info, ["address", "macAddress", "ownerId",
-                            "retailerAccess", "retailerId", "timeZoneId", "id", "hasUserAccount"])
+        pretty_print_except(
+            self.__info,
+            [
+                "address",
+                "macAddress",
+                "ownerId",
+                "retailerAccess",
+                "retailerId",
+                "timeZoneId",
+                "id",
+                "hasUserAccount",
+            ],
+        )
 
         print("self.__status:")
         pretty_print_except(self.__status)
 
         print("self.__device_data:")
-        pretty_print_except(self.__device_data, [
-                            "macAddress", "owner", "retailerAccess", "retailerId", "id", "status"])
+        pretty_print_except(
+            self.__device_data,
+            ["macAddress", "owner", "retailerAccess", "retailerId", "id", "status"],
+        )
 
         print("self.__group_temperatures:")
         pretty_print_except(self.__group_temperatures)
@@ -548,7 +605,8 @@ class ThermiaHeatPump:
 
         if installation_profile_id is not None:
             all_available_groups = self.__api_interface.get_all_available_groups(
-                installation_profile_id)
+                installation_profile_id
+            )
             if all_available_groups is not None:
                 print("All available groups:")
                 pretty_print_except(all_available_groups)
@@ -558,7 +616,8 @@ class ThermiaHeatPump:
                     if group_name is not None:
                         print("Group " + group_name + ":")
                         group_data = self.__api_interface.get_register_group_json(
-                            self.__device_id, group_name)
+                            self.__device_id, group_name
+                        )
                         pretty_print_except(group_data)
 
         print("########## DEBUG END ##########")
